"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _crypto = _interopRequireDefault(require("crypto"));

var _webpack = _interopRequireWildcard(require("webpack"));

var _schemaUtils = require("schema-utils");

var _options = _interopRequireDefault(require("./options.json"));

var _minify = require("./minify");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// webpack 5 exposes the sources property to ensure the right version of webpack-sources is used
const {
  RawSource
} = // eslint-disable-next-line global-require
_webpack.default.sources || require('webpack-sources');

class JsonMinimizerPlugin {
  constructor(options = {}) {
    (0, _schemaUtils.validate)(_options.default, options, {
      name: 'Json Minimizer Plugin',
      baseDataPath: 'options'
    });
    const {
      minimizerOptions = {},
      test = /\.json(\?.*)?$/i,
      cache = true,
      cacheKeys = defaultCacheKeys => defaultCacheKeys,
      include,
      exclude
    } = options;
    this.options = {
      test,
      cache,
      cacheKeys,
      include,
      exclude,
      minimizerOptions
    };
  }

  static buildError(error, file, context) {
    return new Error(`"${file}" in "${context}" from Json Minimizer:\n${error}`);
  } // eslint-disable-next-line consistent-return


  static getAsset(compilation, name) {
    // New API
    if (compilation.getAsset) {
      return compilation.getAsset(name);
    }

    if (compilation.assets[name]) {
      return {
        name,
        source: compilation.assets[name],
        info: {}
      };
    }
  }

  static updateAsset(compilation, name, newSource, assetInfo) {
    // New API
    if (compilation.updateAsset) {
      compilation.updateAsset(name, newSource, assetInfo);
    } // eslint-disable-next-line no-param-reassign


    compilation.assets[name] = newSource;
  }

  async optimize(compiler, compilation, assets, CacheEngine, weakCache) {
    const assetNames = Object.keys(typeof assets === 'undefined' ? compilation.assets : assets).filter(assetName => // eslint-disable-next-line no-undefined
    _webpack.ModuleFilenameHelpers.matchObject.bind(undefined, this.options)(assetName));

    if (assetNames.length === 0) {
      return Promise.resolve();
    }

    const scheduledTasks = [];
    const cache = new CacheEngine(compilation, {
      cache: this.options.cache
    }, weakCache);

    for (const name of assetNames) {
      scheduledTasks.push((async () => {
        const {
          source: inputSource,
          info
        } = JsonMinimizerPlugin.getAsset(compilation, name); // Skip double minimize assets from child compilation

        if (info.minimized) {
          return;
        }

        let input = inputSource.source();

        if (Buffer.isBuffer(input)) {
          input = input.toString();
        }

        const cacheData = {
          inputSource
        };

        if (JsonMinimizerPlugin.isWebpack4()) {
          if (this.options.cache) {
            cacheData.input = input;
            cacheData.cacheKeys = this.options.cacheKeys({
              // eslint-disable-next-line global-require
              'json-minimizer-webpack-plugin': require('../package.json').version,
              'json-minimizer-webpack-plugin-options': this.options,
              name,
              contentHash: _crypto.default.createHash('md4').update(input).digest('hex')
            }, name);
          }
        } else {
          cacheData.name = name;
        }

        let output = await cache.get(cacheData, {
          RawSource
        });

        if (!output) {
          try {
            const minimizerOptions = {
              input,
              minimizerOptions: this.options.minimizerOptions,
              minify: this.options.minify
            };
            output = await (0, _minify.minify)(minimizerOptions);
          } catch (error) {
            compilation.errors.push(JsonMinimizerPlugin.buildError(error, name, compiler.context));
            return;
          }

          output.source = new RawSource(output.output);
          await cache.store({ ...output,
            ...cacheData
          });
        } // TODO `...` required only for webpack@4


        JsonMinimizerPlugin.updateAsset(compilation, name, output.source, { ...info,
          minimized: true
        });
      })());
    }

    return Promise.all(scheduledTasks);
  }

  static isWebpack4() {
    return _webpack.version[0] === '4';
  }

  apply(compiler) {
    const pluginName = this.constructor.name;
    const weakCache = new WeakMap();
    compiler.hooks.compilation.tap(pluginName, compilation => {
      if (JsonMinimizerPlugin.isWebpack4()) {
        // eslint-disable-next-line global-require
        const CacheEngine = require('./Webpack4Cache').default;

        compilation.hooks.optimizeChunkAssets.tapPromise(pluginName, () => {
          return this.optimize(compiler, compilation, // eslint-disable-next-line no-undefined
          undefined, CacheEngine, weakCache);
        });
      } else {
        // eslint-disable-next-line global-require
        const CacheEngine = require('./Webpack5Cache').default; // eslint-disable-next-line global-require


        const Compilation = require('webpack/lib/Compilation');

        compilation.hooks.processAssets.tapPromise({
          name: pluginName,
          stage: Compilation.PROCESS_ASSETS_STAGE_OPTIMIZE_SIZE
        }, assets => this.optimize(compiler, compilation, assets, CacheEngine));
        compilation.hooks.statsPrinter.tap(pluginName, stats => {
          stats.hooks.print.for('asset.info.minimized').tap('json-minimizer-webpack-plugin', (minimized, {
            green,
            formatFlag
          }) => // eslint-disable-next-line no-undefined
          minimized ? green(formatFlag('minimized')) : undefined);
        });
      }
    });
  }

}

var _default = JsonMinimizerPlugin;
exports.default = _default;